@@ -1,13 +1,14 @@
 #[cfg(all(test, feature = "c_bindings"))]
+#[allow(clippy::all)]
 mod gtest {
     use crate::{
         c_bindings::*, iccread::*, transform::DataType::*, transform::*,
         transform_util::lut_inverse_interp16, Intent::Perceptual,
     };
     use libc::c_void;
-    #[cfg(all(target_arch = "arm", std_arch))]
+    #[cfg(target_arch = "arm")]
     use std::arch::is_arm_feature_detected;
-    #[cfg(all(target_arch = "aarch64", std_arch))]
+    #[cfg(target_arch = "aarch64")]
     use std::arch::is_aarch64_feature_detected;
     use std::ptr::null_mut;
 
@@ -161,19 +162,19 @@ mod gtest {
 
         let Rec709Primaries = qcms_CIE_xyYTRIPLE {
             red: qcms_CIE_xyY {
-                x: 0.6400f64,
-                y: 0.3300f64,
-                Y: 1.0f64,
+                x: 0.6400,
+                y: 0.3300,
+                Y: 1.0,
             },
             green: qcms_CIE_xyY {
-                x: 0.3000f64,
-                y: 0.6000f64,
-                Y: 1.0f64,
+                x: 0.3000,
+                y: 0.6000,
+                Y: 1.0,
             },
             blue: qcms_CIE_xyY {
-                x: 0.1500f64,
-                y: 0.0600f64,
-                Y: 1.0f64,
+                x: 0.1500,
+                y: 0.0600,
+                Y: 1.0,
             },
         };
         let D65 = qcms_white_point_sRGB();
@@ -916,7 +917,7 @@ mod test {
         let mut file = std::fs::File::open(path).unwrap();
         let mut data = Vec::new();
         file.read_to_end(&mut data).unwrap();
-        Profile::new_from_slice(&data).unwrap()
+        Profile::new_from_slice(&data, false).unwrap()
     }
 
     #[test]
