@@ -9,16 +9,16 @@
 #ifndef mozilla_ReflowInput_h
 #define mozilla_ReflowInput_h
 
-#include "nsMargin.h"
-#include "nsStyleConsts.h"
-#include "mozilla/Assertions.h"
+#include <algorithm>
+
+#include "LayoutConstants.h"
+#include "ReflowOutput.h"
 #include "mozilla/EnumSet.h"
 #include "mozilla/LayoutStructs.h"
 #include "mozilla/Maybe.h"
 #include "mozilla/WritingModes.h"
-#include "LayoutConstants.h"
-#include "ReflowOutput.h"
-#include <algorithm>
+#include "nsMargin.h"
+#include "nsStyleConsts.h"
 
 class gfxContext;
 class nsFloatManager;
@@ -68,6 +68,9 @@ struct SizeComputationInput {
   // Rendering context to use for measurement.
   gfxContext* mRenderingContext;
 
+  // Cache for anchor resolution in this computation.
+  AnchorPosResolutionCache* mAnchorPosResolutionCache = nullptr;
+
   nsMargin ComputedPhysicalMargin() const {
     return mComputedMargin.GetPhysicalMargin(mWritingMode);
   }
@@ -127,7 +130,9 @@ struct SizeComputationInput {
 
  public:
   // Callers using this constructor must call InitOffsets on their own.
-  SizeComputationInput(nsIFrame* aFrame, gfxContext* aRenderingContext);
+  SizeComputationInput(
+      nsIFrame* aFrame, gfxContext* aRenderingContext,
+      AnchorPosResolutionCache* aAnchorPosResolutionCache = nullptr);
 
   SizeComputationInput(nsIFrame* aFrame, gfxContext* aRenderingContext,
                        WritingMode aContainingBlockWritingMode,
@@ -501,6 +506,11 @@ struct ReflowInput : public SizeComputationInput {
     bool mIOffsetsNeedCSSAlign : 1;
     bool mBOffsetsNeedCSSAlign : 1;
 
+    // True when anchor-center is being used with a valid anchor and at least
+    // one inset is auto on this axis. Used to zero out margins.
+    bool mIAnchorCenter : 1;
+    bool mBAnchorCenter : 1;
+
     // Is this frame or one of its ancestors being reflowed in a different
     // continuation than the one in which it was previously reflowed?  In
     // other words, has it moved to a different column or page than it was in
@@ -522,6 +532,14 @@ struct ReflowInput : public SizeComputationInput {
     // If true, then children of this frame can generate class A breakpoints
     // for paginated reflow.
     bool mCanHaveClassABreakpoints : 1;
+
+    // If set:
+    // (1) This frame is absolutely-positioned,
+    // (2) Inset in that axis are non-auto, and
+    // (3) Size in that axis is `auto` & resolved as fit-content size.
+    // Automatic margin computation in this case requires waiting until
+    // the frame reflows to compute the fit-content size.
+    bool mDeferAutoMarginComputation : 1;
   };
   Flags mFlags;
 
@@ -613,6 +631,9 @@ struct ReflowInput : public SizeComputationInput {
    *        call nsIFrame::ComputeSize() internally.
    * @param aComputeSizeFlags A set of flags used when we call
    *        nsIFrame::ComputeSize() internally.
+   * @param aAnchorResolutionCache A cache of referenced anchors to be populated
+   *        (If specified) for this reflowed frame. Should live for the lifetime
+   *        of this ReflowInput.
    */
   ReflowInput(nsPresContext* aPresContext,
               const ReflowInput& aParentReflowInput, nsIFrame* aFrame,
@@ -620,7 +641,8 @@ struct ReflowInput : public SizeComputationInput {
               const Maybe<LogicalSize>& aContainingBlockSize = Nothing(),
               InitFlags aFlags = {},
               const StyleSizeOverrides& aSizeOverrides = {},
-              ComputeSizeFlags aComputeSizeFlags = {});
+              ComputeSizeFlags aComputeSizeFlags = {},
+              AnchorPosResolutionCache* aAnchorPosResolutionCache = nullptr);
 
   /**
    * This method initializes various data members. It is automatically called by
@@ -683,9 +705,6 @@ struct ReflowInput : public SizeComputationInput {
 
   static constexpr float kNormalLineHeightFactor = 1.2f;
 
-  LogicalSize ComputeContainingBlockRectangle(
-      nsPresContext* aPresContext, const ReflowInput* aContainingBlockRI) const;
-
   /**
    * Apply the mComputed(Min/Max)ISize constraints to the content
    * size computed so far.
@@ -813,8 +832,8 @@ struct ReflowInput : public SizeComputationInput {
                                            WritingMode aContainingBlockWM,
                                            bool aIsMarginBStartAuto,
                                            bool aIsMarginBEndAuto,
-                                           LogicalMargin& aMargin,
-                                           LogicalMargin& aOffsets);
+                                           bool aIsIAnchorCenter,
+                                           LogicalMargin& aMargin);
 
   // Resolve any inline-axis 'auto' margins (if any) for an absolutely
   // positioned frame. aMargin and aOffsets are both outparams (though we only
@@ -823,8 +842,8 @@ struct ReflowInput : public SizeComputationInput {
                                             WritingMode aContainingBlockWM,
                                             bool aIsMarginIStartAuto,
                                             bool aIsMarginIEndAuto,
-                                            LogicalMargin& aMargin,
-                                            LogicalMargin& aOffsets);
+                                            bool aIsBAnchorCenter,
+                                            LogicalMargin& aMargin);
 
  protected:
   void InitCBReflowInput();
@@ -837,14 +856,25 @@ struct ReflowInput : public SizeComputationInput {
                        const Maybe<LogicalMargin>& aPadding,
                        LayoutFrameType aFrameType);
 
-  // Returns the nearest containing block or block frame (whether or not
-  // it is a containing block) for the specified frame.  Also returns
-  // the inline-start edge and logical size of the containing block's
-  // content area.
-  // These are returned in the coordinate space of the containing block.
-  nsIFrame* GetHypotheticalBoxContainer(nsIFrame* aFrame,
-                                        nscoord& aCBIStartEdge,
-                                        LogicalSize& aCBSize) const;
+  // Compute the content-box size of the containing block frame in mFrame's
+  // writing-mode (mWritingMode).
+  //
+  // Note: the block-size in the return value may be unconstrained.
+  LogicalSize ComputeContainingBlockRectangle(
+      nsPresContext* aPresContext, const ReflowInput* aContainingBlockRI) const;
+
+  // mBorderPadding and mFrame are both in mBoxContainer's writing-mode.
+  struct HypotheticalBoxContainerInfo {
+    nsIFrame* mBoxContainer;
+    LogicalMargin mBorderPadding;
+    LogicalSize mContentBoxSize;
+  };
+
+  // Returns the nearest containing block for aFrame. Also returns its border &
+  // padding and content-box size. These are returned in the coordinate space of
+  // the containing block.
+  HypotheticalBoxContainerInfo GetHypotheticalBoxContainer(
+      const nsIFrame* aFrame) const;
 
   // Calculate the position of the hypothetical box that the placeholder frame
   // (for a position:fixed/absolute element) would have if it were in the flow
@@ -854,8 +884,12 @@ struct ReflowInput : public SizeComputationInput {
   // absolute containing block (aCBReflowInput->mFrame). The writing mode of the
   // hypothetical box will have the same block direction as the absolute
   // containing block, but it may differ in the inline direction.
+  //
+  // @param aCBPaddingBoxSize the padding-box size of the absolute containing
+  // block, in its own writing-mode.
   void CalculateHypotheticalPosition(
       nsPlaceholderFrame* aPlaceholderFrame, const ReflowInput* aCBReflowInput,
+      const LogicalSize& aCBPaddingBoxSize,
       nsHypotheticalPosition& aHypotheticalPos) const;
 
   void InitAbsoluteConstraints(const ReflowInput* aCBReflowInput,
@@ -950,9 +984,9 @@ struct ReflowInput : public SizeComputationInput {
 
 }  // namespace mozilla
 
-inline AnchorPosResolutionParams AnchorPosResolutionParams::From(
-    const mozilla::ReflowInput* aRI) {
-  return {aRI->mFrame, aRI->mStyleDisplay->mPosition};
-}
+void ComputeAnchorCenterUsage(
+    const nsIFrame* aFrame,
+    mozilla::AnchorPosResolutionCache* aAnchorPosResolutionCache,
+    bool& aInlineUsesAnchorCenter, bool& aBlockUsesAnchorCenter);
 
 #endif  // mozilla_ReflowInput_h
