@@ -4,8 +4,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "ErrorList.h"
 #include "SelectionMovementUtils.h"
+
+#include "ErrorList.h"
+#include "WordMovementType.h"
 #include "mozilla/CaretAssociationHint.h"
 #include "mozilla/Maybe.h"
 #include "mozilla/PresShell.h"
@@ -16,8 +18,8 @@
 #include "mozilla/intl/BidiEmbeddingLevel.h"
 #include "nsBidiPresUtils.h"
 #include "nsBlockFrame.h"
-#include "nsCaret.h"
 #include "nsCOMPtr.h"
+#include "nsCaret.h"
 #include "nsFrameSelection.h"
 #include "nsFrameTraversal.h"
 #include "nsIContent.h"
@@ -26,7 +28,6 @@
 #include "nsLayoutUtils.h"
 #include "nsPresContext.h"
 #include "nsTextFrame.h"
-#include "WordMovementType.h"
 
 namespace mozilla {
 using namespace dom;
@@ -114,7 +115,7 @@ SelectionMovementUtils::PeekOffsetForCaretMove(
 // static
 nsPrevNextBidiLevels SelectionMovementUtils::GetPrevNextBidiLevels(
     nsIContent* aNode, uint32_t aContentOffset, CaretAssociationHint aHint,
-    bool aJumpLines) {
+    bool aJumpLines, const Element* aAncestorLimiter) {
   // Get the level of the frames on each side
   nsIFrame* currentFrame;
   uint32_t currentOffset;
@@ -150,8 +151,10 @@ nsPrevNextBidiLevels SelectionMovementUtils::GetPrevNextBidiLevels(
   if (aJumpLines) {
     peekOffsetOptions += PeekOffsetOption::JumpLines;
   }
-  nsIFrame* newFrame =
-      currentFrame->GetFrameFromDirection(direction, peekOffsetOptions).mFrame;
+  nsIFrame* newFrame = currentFrame
+                           ->GetFrameFromDirection(direction, peekOffsetOptions,
+                                                   aAncestorLimiter)
+                           .mFrame;
 
   FrameBidiData currentBidi = currentFrame->GetBidiData();
   intl::BidiEmbeddingLevel currentLevel = currentBidi.embeddingLevel;
@@ -259,7 +262,7 @@ nsIFrame* SelectionMovementUtils::GetFrameForNodeOffset(
 
   nsIFrame *returnFrame = nullptr, *lastFrame = aNode->GetPrimaryFrame();
   nsCOMPtr<nsIContent> theNode;
-  uint32_t offsetInFrameContent, offsetInLastFrameContent = 0;
+  uint32_t offsetInFrameContent, offsetInLastFrameContent = aOffset;
 
   while (true) {
     if (returnFrame) {
@@ -553,7 +556,11 @@ CaretFrameData SelectionMovementUtils::GetCaretFrameForNodeOffset(
       static_cast<uint32_t>(start) == result.mOffsetInFrameContent ||
       static_cast<uint32_t>(end) == result.mOffsetInFrameContent) {
     nsPrevNextBidiLevels levels = SelectionMovementUtils::GetPrevNextBidiLevels(
-        aContentNode, aOffset, result.mHint, false);
+        aContentNode, aOffset, result.mHint, false,
+        aFrameSelection
+            ? aFrameSelection
+                  ->GetAncestorLimiterOrIndependentSelectionRootElement()
+            : nullptr);
 
     /* Boundary condition, we need to know the Bidi levels of the characters
      * before and after the caret */
