@@ -7,64 +7,42 @@
 #include "nsFrameList.h"
 
 #include "mozilla/ArenaObjectID.h"
+#include "mozilla/PresShell.h"
+#include "mozilla/intl/BidiEmbeddingLevel.h"
 #include "nsBidiPresUtils.h"
 #include "nsContainerFrame.h"
 #include "nsGkAtoms.h"
 #include "nsILineIterator.h"
-#include "nsIPresShell.h"
 #include "nsLayoutUtils.h"
 #include "nsPresContext.h"
 
 using namespace mozilla;
 
-namespace mozilla {
-namespace layout {
-namespace detail {
-const AlignedFrameListBytes gEmptyFrameListBytes = {0};
-}  // namespace detail
-}  // namespace layout
-}  // namespace mozilla
+const nsFrameList nsFrameList::sEmptyList;
 
-void* nsFrameList::operator new(size_t sz, nsIPresShell* aPresShell) {
+void* nsFrameList::operator new(size_t sz, mozilla::PresShell* aPresShell) {
   return aPresShell->AllocateByObjectID(eArenaObjectID_nsFrameList, sz);
 }
 
-void nsFrameList::Delete(nsIPresShell* aPresShell) {
-  NS_PRECONDITION(this != &EmptyList(), "Shouldn't Delete() this list");
+void nsFrameList::Delete(mozilla::PresShell* aPresShell) {
+  MOZ_ASSERT(this != &EmptyList(), "Shouldn't Delete() this list");
   NS_ASSERTION(IsEmpty(), "Shouldn't Delete() a non-empty list");
 
   aPresShell->FreeByObjectID(eArenaObjectID_nsFrameList, this);
 }
 
-void nsFrameList::DestroyFrames() {
-  while (nsIFrame* frame = RemoveFirstChild()) {
-    frame->Destroy();
+void nsFrameList::DestroyFrames(FrameDestroyContext& aContext) {
+  while (nsIFrame* frame = RemoveLastChild()) {
+    frame->Destroy(aContext);
   }
-  mLastChild = nullptr;
-}
-
-void nsFrameList::DestroyFramesFrom(
-    nsIFrame* aDestructRoot, layout::PostFrameDestroyData& aPostDestroyData) {
-  NS_PRECONDITION(aDestructRoot, "Missing destruct root");
-
-  while (nsIFrame* frame = RemoveFirstChild()) {
-    frame->DestroyFrom(aDestructRoot, aPostDestroyData);
-  }
-  mLastChild = nullptr;
-}
-
-void nsFrameList::SetFrames(nsIFrame* aFrameList) {
-  NS_PRECONDITION(!mFirstChild, "Losing frames");
-
-  mFirstChild = aFrameList;
-  mLastChild = nsLayoutUtils::GetLastSibling(mFirstChild);
+  MOZ_ASSERT(!mFirstChild && !mLastChild, "We should've destroyed all frames!");
 }
 
 void nsFrameList::RemoveFrame(nsIFrame* aFrame) {
-  NS_PRECONDITION(aFrame, "null ptr");
+  MOZ_ASSERT(aFrame, "null ptr");
 #ifdef DEBUG_FRAME_LIST
   // ContainsFrame is O(N)
-  NS_PRECONDITION(ContainsFrame(aFrame), "wrong list");
+  MOZ_ASSERT(ContainsFrame(aFrame), "wrong list");
 #endif
 
   nsIFrame* nextFrame = aFrame->GetNextSibling();
@@ -86,24 +64,22 @@ void nsFrameList::RemoveFrame(nsIFrame* aFrame) {
   }
 }
 
-nsFrameList nsFrameList::RemoveFramesAfter(nsIFrame* aAfterFrame) {
-  if (!aAfterFrame) {
-    nsFrameList result;
-    result.InsertFrames(nullptr, nullptr, *this);
-    return result;
+nsFrameList nsFrameList::TakeFramesAfter(nsIFrame* aFrame) {
+  if (!aFrame) {
+    return std::move(*this);
   }
 
-  NS_PRECONDITION(NotEmpty(), "illegal operation on empty list");
-#ifdef DEBUG_FRAME_LIST
-  NS_PRECONDITION(ContainsFrame(aAfterFrame), "wrong list");
-#endif
+  MOZ_ASSERT(ContainsFrame(aFrame), "aFrame is not on this list!");
 
-  nsIFrame* tail = aAfterFrame->GetNextSibling();
-  // if (!tail) return EmptyList();  -- worth optimizing this case?
-  nsIFrame* oldLastChild = mLastChild;
-  mLastChild = aAfterFrame;
-  aAfterFrame->SetNextSibling(nullptr);
-  return nsFrameList(tail, tail ? oldLastChild : nullptr);
+  nsIFrame* newFirstChild = aFrame->GetNextSibling();
+  if (!newFirstChild) {
+    return nsFrameList();
+  }
+
+  nsIFrame* newLastChild = mLastChild;
+  mLastChild = aFrame;
+  mLastChild->SetNextSibling(nullptr);
+  return nsFrameList(newFirstChild, newLastChild);
 }
 
 nsIFrame* nsFrameList::RemoveFirstChild() {
@@ -115,16 +91,26 @@ nsIFrame* nsFrameList::RemoveFirstChild() {
   return nullptr;
 }
 
-void nsFrameList::DestroyFrame(nsIFrame* aFrame) {
-  NS_PRECONDITION(aFrame, "null ptr");
+nsIFrame* nsFrameList::RemoveLastChild() {
+  if (mLastChild) {
+    nsIFrame* lastChild = mLastChild;
+    RemoveFrame(lastChild);
+    return lastChild;
+  }
+  return nullptr;
+}
+
+void nsFrameList::DestroyFrame(FrameDestroyContext& aContext,
+                               nsIFrame* aFrame) {
+  MOZ_ASSERT(aFrame, "null ptr");
   RemoveFrame(aFrame);
-  aFrame->Destroy();
+  aFrame->Destroy(aContext);
 }
 
 nsFrameList::Slice nsFrameList::InsertFrames(nsContainerFrame* aParent,
                                              nsIFrame* aPrevSibling,
-                                             nsFrameList& aFrameList) {
-  NS_PRECONDITION(aFrameList.NotEmpty(), "Unexpected empty list");
+                                             nsFrameList&& aFrameList) {
+  MOZ_ASSERT(aFrameList.NotEmpty(), "Unexpected empty list");
 
   if (aParent) {
     aFrameList.ApplySetParent(aParent);
@@ -161,79 +147,38 @@ nsFrameList::Slice nsFrameList::InsertFrames(nsContainerFrame* aParent,
   VerifyList();
 
   aFrameList.Clear();
-  return Slice(*this, firstNewFrame, nextSibling);
+  return Slice(firstNewFrame, nextSibling);
 }
 
-nsFrameList nsFrameList::ExtractHead(FrameLinkEnumerator& aLink) {
-  NS_PRECONDITION(&aLink.List() == this, "Unexpected list");
-  NS_PRECONDITION(!aLink.PrevFrame() ||
-                      aLink.PrevFrame()->GetNextSibling() == aLink.NextFrame(),
-                  "Unexpected PrevFrame()");
-  NS_PRECONDITION(aLink.PrevFrame() || aLink.NextFrame() == FirstChild(),
-                  "Unexpected NextFrame()");
-  NS_PRECONDITION(!aLink.PrevFrame() || aLink.NextFrame() != FirstChild(),
-                  "Unexpected NextFrame()");
-  NS_PRECONDITION(aLink.mEnd == nullptr,
-                  "Unexpected mEnd for frame link enumerator");
-
-  nsIFrame* prev = aLink.PrevFrame();
-  nsIFrame* newFirstFrame = nullptr;
-  if (prev) {
-    // Truncate the list after |prev| and hand the first part to our new list.
-    prev->SetNextSibling(nullptr);
-    newFirstFrame = mFirstChild;
-    mFirstChild = aLink.NextFrame();
-    if (!mFirstChild) {  // we handed over the whole list
-      mLastChild = nullptr;
-    }
-
-    // Now make sure aLink doesn't point to a frame we no longer have.
-    aLink.mPrev = nullptr;
-  }
-  // else aLink is pointing to before our first frame.  Nothing to do.
-
-  return nsFrameList(newFirstFrame, prev);
-}
-
-nsFrameList nsFrameList::ExtractTail(FrameLinkEnumerator& aLink) {
-  NS_PRECONDITION(&aLink.List() == this, "Unexpected list");
-  NS_PRECONDITION(!aLink.PrevFrame() ||
-                      aLink.PrevFrame()->GetNextSibling() == aLink.NextFrame(),
-                  "Unexpected PrevFrame()");
-  NS_PRECONDITION(aLink.PrevFrame() || aLink.NextFrame() == FirstChild(),
-                  "Unexpected NextFrame()");
-  NS_PRECONDITION(!aLink.PrevFrame() || aLink.NextFrame() != FirstChild(),
-                  "Unexpected NextFrame()");
-  NS_PRECONDITION(aLink.mEnd == nullptr,
-                  "Unexpected mEnd for frame link enumerator");
-
-  nsIFrame* prev = aLink.PrevFrame();
-  nsIFrame* newFirstFrame;
-  nsIFrame* newLastFrame;
-  if (prev) {
-    // Truncate the list after |prev| and hand the second part to our new list
-    prev->SetNextSibling(nullptr);
-    newFirstFrame = aLink.NextFrame();
-    newLastFrame = newFirstFrame ? mLastChild : nullptr;
-    mLastChild = prev;
-  } else {
-    // Hand the whole list over to our new list
-    newFirstFrame = mFirstChild;
-    newLastFrame = mLastChild;
-    Clear();
+nsFrameList nsFrameList::TakeFramesBefore(nsIFrame* aFrame) {
+  if (!aFrame) {
+    // We handed over the whole list.
+    return std::move(*this);
   }
 
-  // Now make sure aLink doesn't point to a frame we no longer have.
-  aLink.mFrame = nullptr;
+  MOZ_ASSERT(ContainsFrame(aFrame), "aFrame is not on this list!");
 
-  MOZ_ASSERT(aLink.AtEnd(), "What's going on here?");
+  if (aFrame == mFirstChild) {
+    // aFrame is our first child. Nothing to extract.
+    return nsFrameList();
+  }
 
-  return nsFrameList(newFirstFrame, newLastFrame);
+  // Extract all previous siblings of aFrame as a new list.
+  nsIFrame* prev = aFrame->GetPrevSibling();
+  nsIFrame* newFirstChild = mFirstChild;
+  nsIFrame* newLastChild = prev;
+
+  prev->SetNextSibling(nullptr);
+  mFirstChild = aFrame;
+
+  return nsFrameList(newFirstChild, newLastChild);
 }
 
 nsIFrame* nsFrameList::FrameAt(int32_t aIndex) const {
-  NS_PRECONDITION(aIndex >= 0, "invalid arg");
-  if (aIndex < 0) return nullptr;
+  MOZ_ASSERT(aIndex >= 0, "invalid arg");
+  if (aIndex < 0) {
+    return nullptr;
+  }
   nsIFrame* frame = mFirstChild;
   while ((aIndex-- > 0) && frame) {
     frame = frame->GetNextSibling();
@@ -241,17 +186,19 @@ nsIFrame* nsFrameList::FrameAt(int32_t aIndex) const {
   return frame;
 }
 
-int32_t nsFrameList::IndexOf(nsIFrame* aFrame) const {
+int32_t nsFrameList::IndexOf(const nsIFrame* aFrame) const {
   int32_t count = 0;
   for (nsIFrame* f = mFirstChild; f; f = f->GetNextSibling()) {
-    if (f == aFrame) return count;
+    if (f == aFrame) {
+      return count;
+    }
     ++count;
   }
   return -1;
 }
 
 bool nsFrameList::ContainsFrame(const nsIFrame* aFrame) const {
-  NS_PRECONDITION(aFrame, "null ptr");
+  MOZ_ASSERT(aFrame, "null ptr");
 
   nsIFrame* frame = mFirstChild;
   while (frame) {
@@ -281,7 +228,8 @@ void nsFrameList::ApplySetParent(nsContainerFrame* aParent) const {
   }
 }
 
-/* static */ void nsFrameList::UnhookFrameFromSiblings(nsIFrame* aFrame) {
+/* static */
+void nsFrameList::UnhookFrameFromSiblings(nsIFrame* aFrame) {
   MOZ_ASSERT(aFrame->GetPrevSibling() && aFrame->GetNextSibling());
   nsIFrame* const nextSibling = aFrame->GetNextSibling();
   nsIFrame* const prevSibling = aFrame->GetPrevSibling();
@@ -301,20 +249,26 @@ void nsFrameList::List(FILE* out) const {
 #endif
 
 nsIFrame* nsFrameList::GetPrevVisualFor(nsIFrame* aFrame) const {
-  if (!mFirstChild) return nullptr;
+  if (!mFirstChild) {
+    return nullptr;
+  }
 
   nsIFrame* parent = mFirstChild->GetParent();
-  if (!parent) return aFrame ? aFrame->GetPrevSibling() : LastChild();
+  if (!parent) {
+    return aFrame ? aFrame->GetPrevSibling() : LastChild();
+  }
 
-  nsBidiDirection paraDir = nsBidiPresUtils::ParagraphDirection(mFirstChild);
+  mozilla::intl::BidiDirection paraDir =
+      nsBidiPresUtils::ParagraphDirection(mFirstChild);
 
-  nsAutoLineIterator iter = parent->GetLineIterator();
+  AutoAssertNoDomMutations guard;
+  nsILineIterator* iter = parent->GetLineIterator();
   if (!iter) {
     // Parent is not a block Frame
     if (parent->IsLineFrame()) {
       // Line frames are not bidi-splittable, so need to consider bidi
       // reordering
-      if (paraDir == NSBIDI_LTR) {
+      if (paraDir == mozilla::intl::BidiDirection::LTR) {
         return nsBidiPresUtils::GetFrameToLeftOf(aFrame, mFirstChild, -1);
       } else {  // RTL
         return nsBidiPresUtils::GetFrameToRightOf(aFrame, mFirstChild, -1);
@@ -336,59 +290,63 @@ nsIFrame* nsFrameList::GetPrevVisualFor(nsIFrame* aFrame) const {
   int32_t thisLine;
   if (aFrame) {
     thisLine = iter->FindLineContaining(aFrame);
-    if (thisLine < 0) return nullptr;
+    if (thisLine < 0) {
+      return nullptr;
+    }
   } else {
     thisLine = iter->GetNumLines();
   }
 
   nsIFrame* frame = nullptr;
-  nsIFrame* firstFrameOnLine;
-  int32_t numFramesOnLine;
-  nsRect lineBounds;
 
   if (aFrame) {
-    iter->GetLine(thisLine, &firstFrameOnLine, &numFramesOnLine, lineBounds);
+    auto line = iter->GetLine(thisLine).unwrap();
 
-    if (paraDir == NSBIDI_LTR) {
-      frame = nsBidiPresUtils::GetFrameToLeftOf(aFrame, firstFrameOnLine,
-                                                numFramesOnLine);
+    if (paraDir == mozilla::intl::BidiDirection::LTR) {
+      frame = nsBidiPresUtils::GetFrameToLeftOf(aFrame, line.mFirstFrameOnLine,
+                                                line.mNumFramesOnLine);
     } else {  // RTL
-      frame = nsBidiPresUtils::GetFrameToRightOf(aFrame, firstFrameOnLine,
-                                                 numFramesOnLine);
+      frame = nsBidiPresUtils::GetFrameToRightOf(aFrame, line.mFirstFrameOnLine,
+                                                 line.mNumFramesOnLine);
     }
   }
 
   if (!frame && thisLine > 0) {
     // Get the last frame of the previous line
-    iter->GetLine(thisLine - 1, &firstFrameOnLine, &numFramesOnLine,
-                  lineBounds);
+    auto line = iter->GetLine(thisLine - 1).unwrap();
 
-    if (paraDir == NSBIDI_LTR) {
-      frame = nsBidiPresUtils::GetFrameToLeftOf(nullptr, firstFrameOnLine,
-                                                numFramesOnLine);
+    if (paraDir == mozilla::intl::BidiDirection::LTR) {
+      frame = nsBidiPresUtils::GetFrameToLeftOf(nullptr, line.mFirstFrameOnLine,
+                                                line.mNumFramesOnLine);
     } else {  // RTL
-      frame = nsBidiPresUtils::GetFrameToRightOf(nullptr, firstFrameOnLine,
-                                                 numFramesOnLine);
+      frame = nsBidiPresUtils::GetFrameToRightOf(
+          nullptr, line.mFirstFrameOnLine, line.mNumFramesOnLine);
     }
   }
   return frame;
 }
 
 nsIFrame* nsFrameList::GetNextVisualFor(nsIFrame* aFrame) const {
-  if (!mFirstChild) return nullptr;
+  if (!mFirstChild) {
+    return nullptr;
+  }
 
   nsIFrame* parent = mFirstChild->GetParent();
-  if (!parent) return aFrame ? aFrame->GetPrevSibling() : mFirstChild;
+  if (!parent) {
+    return aFrame ? aFrame->GetPrevSibling() : mFirstChild;
+  }
 
-  nsBidiDirection paraDir = nsBidiPresUtils::ParagraphDirection(mFirstChild);
+  mozilla::intl::BidiDirection paraDir =
+      nsBidiPresUtils::ParagraphDirection(mFirstChild);
 
-  nsAutoLineIterator iter = parent->GetLineIterator();
+  AutoAssertNoDomMutations guard;
+  nsILineIterator* iter = parent->GetLineIterator();
   if (!iter) {
     // Parent is not a block Frame
     if (parent->IsLineFrame()) {
       // Line frames are not bidi-splittable, so need to consider bidi
       // reordering
-      if (paraDir == NSBIDI_LTR) {
+      if (paraDir == mozilla::intl::BidiDirection::LTR) {
         return nsBidiPresUtils::GetFrameToRightOf(aFrame, mFirstChild, -1);
       } else {  // RTL
         return nsBidiPresUtils::GetFrameToLeftOf(aFrame, mFirstChild, -1);
@@ -410,40 +368,38 @@ nsIFrame* nsFrameList::GetNextVisualFor(nsIFrame* aFrame) const {
   int32_t thisLine;
   if (aFrame) {
     thisLine = iter->FindLineContaining(aFrame);
-    if (thisLine < 0) return nullptr;
+    if (thisLine < 0) {
+      return nullptr;
+    }
   } else {
     thisLine = -1;
   }
 
   nsIFrame* frame = nullptr;
-  nsIFrame* firstFrameOnLine;
-  int32_t numFramesOnLine;
-  nsRect lineBounds;
 
   if (aFrame) {
-    iter->GetLine(thisLine, &firstFrameOnLine, &numFramesOnLine, lineBounds);
+    auto line = iter->GetLine(thisLine).unwrap();
 
-    if (paraDir == NSBIDI_LTR) {
-      frame = nsBidiPresUtils::GetFrameToRightOf(aFrame, firstFrameOnLine,
-                                                 numFramesOnLine);
+    if (paraDir == mozilla::intl::BidiDirection::LTR) {
+      frame = nsBidiPresUtils::GetFrameToRightOf(aFrame, line.mFirstFrameOnLine,
+                                                 line.mNumFramesOnLine);
     } else {  // RTL
-      frame = nsBidiPresUtils::GetFrameToLeftOf(aFrame, firstFrameOnLine,
-                                                numFramesOnLine);
+      frame = nsBidiPresUtils::GetFrameToLeftOf(aFrame, line.mFirstFrameOnLine,
+                                                line.mNumFramesOnLine);
     }
   }
 
   int32_t numLines = iter->GetNumLines();
   if (!frame && thisLine < numLines - 1) {
     // Get the first frame of the next line
-    iter->GetLine(thisLine + 1, &firstFrameOnLine, &numFramesOnLine,
-                  lineBounds);
+    auto line = iter->GetLine(thisLine + 1).unwrap();
 
-    if (paraDir == NSBIDI_LTR) {
-      frame = nsBidiPresUtils::GetFrameToRightOf(nullptr, firstFrameOnLine,
-                                                 numFramesOnLine);
+    if (paraDir == mozilla::intl::BidiDirection::LTR) {
+      frame = nsBidiPresUtils::GetFrameToRightOf(
+          nullptr, line.mFirstFrameOnLine, line.mNumFramesOnLine);
     } else {  // RTL
-      frame = nsBidiPresUtils::GetFrameToLeftOf(nullptr, firstFrameOnLine,
-                                                numFramesOnLine);
+      frame = nsBidiPresUtils::GetFrameToLeftOf(nullptr, line.mFirstFrameOnLine,
+                                                line.mNumFramesOnLine);
     }
   }
   return frame;
@@ -493,7 +449,44 @@ void nsFrameList::VerifyList() const {
 #endif
 
 namespace mozilla {
-namespace layout {
+
+#ifdef DEBUG_FRAME_DUMP
+const char* ChildListName(FrameChildListID aListID) {
+  switch (aListID) {
+    case FrameChildListID::Principal:
+      return "";
+    case FrameChildListID::ColGroup:
+      return "ColGroupList";
+    case FrameChildListID::Absolute:
+      return "AbsoluteList";
+    case FrameChildListID::PushedAbsolute:
+      return "PushedAbsoluteList";
+    case FrameChildListID::Fixed:
+      return "FixedList";
+    case FrameChildListID::Overflow:
+      return "OverflowList";
+    case FrameChildListID::OverflowContainers:
+      return "OverflowContainersList";
+    case FrameChildListID::ExcessOverflowContainers:
+      return "ExcessOverflowContainersList";
+    case FrameChildListID::OverflowOutOfFlow:
+      return "OverflowOutOfFlowList";
+    case FrameChildListID::Float:
+      return "FloatList";
+    case FrameChildListID::Marker:
+      return "MarkerList";
+    case FrameChildListID::PushedFloats:
+      return "PushedFloatsList";
+    case FrameChildListID::Backdrop:
+      return "BackdropList";
+    case FrameChildListID::NoReflowPrincipal:
+      return "NoReflowPrincipalList";
+  }
+
+  MOZ_ASSERT_UNREACHABLE("unknown list");
+  return "UNKNOWN_FRAME_CHILD_LIST";
+}
+#endif
 
 AutoFrameListPtr::~AutoFrameListPtr() {
   if (mFrameList) {
@@ -501,5 +494,4 @@ AutoFrameListPtr::~AutoFrameListPtr() {
   }
 }
 
-}  // namespace layout
 }  // namespace mozilla
