@@ -10,9 +10,9 @@
 #define nsGridContainerFrame_h___
 
 #include "mozilla/CSSOrderAwareFrameIterator.h"
+#include "mozilla/HashTable.h"
 #include "mozilla/IntrinsicISizesCache.h"
 #include "mozilla/Maybe.h"
-#include "mozilla/HashTable.h"
 #include "nsAtomHashKeys.h"
 #include "nsContainerFrame.h"
 #include "nsILineIterator.h"
@@ -161,7 +161,8 @@ class nsGridContainerFrame final : public nsContainerFrame,
   int32_t GetNumLines() const final;
   bool IsLineIteratorFlowRTL() final;
   mozilla::Result<LineInfo, nsresult> GetLine(int32_t aLineNumber) final;
-  int32_t FindLineContaining(nsIFrame* aFrame, int32_t aStartLine = 0) final;
+  int32_t FindLineContaining(const nsIFrame* aFrame,
+                             int32_t aStartLine = 0) final;
   NS_IMETHOD FindFrameAt(int32_t aLineNumber, nsPoint aPos,
                          nsIFrame** aFrameFound, bool* aPosIsBeforeFirstFrame,
                          bool* aPosIsAfterLastFrame) final;
@@ -239,6 +240,18 @@ class nsGridContainerFrame final : public nsContainerFrame,
 
   using nsContainerFrame::IsMasonry;
 
+  /**
+   * Return true if this frame has masonry layout in aAxis (in this frame's own
+   * writing mode).
+   */
+  bool IsMasonry(mozilla::LogicalAxis aAxis) const;
+  bool IsColMasonry() const {
+    return HasAnyStateBits(NS_STATE_GRID_IS_COL_MASONRY);
+  }
+  bool IsRowMasonry() const {
+    return HasAnyStateBits(NS_STATE_GRID_IS_ROW_MASONRY);
+  }
+
   /** Return true if this frame has masonry layout in any axis. */
   bool IsMasonry() const {
     return HasAnyStateBits(NS_STATE_GRID_IS_ROW_MASONRY |
@@ -324,6 +337,8 @@ class nsGridContainerFrame final : public nsContainerFrame,
     // Does the above item span the first(last) track?
     bool mIsInEdgeTrack;
   };
+  class TrackPlan;
+  class ItemPlan;
 
   /** Return our parent grid container; |this| MUST be a subgrid. */
   nsGridContainerFrame* ParentGridContainerForSubgrid() const;
@@ -563,8 +578,7 @@ class nsGridContainerFrame final : public nsContainerFrame,
   UsedTrackSizes* GetUsedTrackSizes() const;
 
   // Store the given TrackSizes in aAxis on a UsedTrackSizes frame property.
-  void StoreUsedTrackSizes(LogicalAxis aAxis,
-                           const nsTArray<TrackSize>& aSizes);
+  void StoreUsedTrackSizes(LogicalAxis aAxis, const TrackPlan& aSizes);
 
   // The internal implementation for AddImplicitNamedAreas().
   void AddImplicitNamedAreasInternal(LineNameList& aNameList,
