@@ -11,14 +11,14 @@
 
 #include "gfxBlur.h"
 #include "gfxContext.h"
+#include "gfxTextRun.h"
+#include "mozilla/TypedEnumBits.h"
 #include "mozilla/gfx/PathHelpers.h"
 #include "mozilla/gfx/Rect.h"
-#include "mozilla/TypedEnumBits.h"
-#include "nsStyleStruct.h"
+#include "nsCSSRenderingBorders.h"
 #include "nsIFrame.h"
 #include "nsImageRenderer.h"
-#include "nsCSSRenderingBorders.h"
-#include "gfxTextRun.h"
+#include "nsStyleStruct.h"
 
 class gfxContext;
 class nsPresContext;
@@ -153,7 +153,7 @@ struct nsCSSRendering {
                                   const nsRect& aDirtyRect,
                                   float aOpacity = 1.0);
 
-  static void ComputePixelRadii(const nscoord* aAppUnitsRadii,
+  static void ComputePixelRadii(const nsRectCornerRadii& aRadii,
                                 nscoord aAppUnitsPerPixel,
                                 RectCornerRadii* oBorderRadii);
 
@@ -278,20 +278,16 @@ struct nsCSSRendering {
   static nsIFrame* FindBackgroundStyleFrame(nsIFrame* aForFrame);
 
   /**
-   * @return true if |aFrame| is a canvas frame, in the CSS sense.
+   * Returns the ComputedStyle to be used to paint the background for the given
+   * frame, if its element has a meaningful background.  This applies the rules
+   * for propagating backgrounds between BODY, the root element, and the
+   * canvas.
+   *
+   * @return the ComputedStyle (if any) to be used for painting aForFrame's
+   *         background.
    */
-  static bool IsCanvasFrame(const nsIFrame* aFrame);
-
-  /**
-   * Fill in an aBackgroundSC to be used to paint the background
-   * for an element.  This applies the rules for propagating
-   * backgrounds between BODY, the root element, and the canvas.
-   * @return true if there is some meaningful background.
-   */
-  static bool FindBackground(const nsIFrame* aForFrame,
-                             mozilla::ComputedStyle** aBackgroundSC);
-  static bool FindBackgroundFrame(const nsIFrame* aForFrame,
-                                  nsIFrame** aBackgroundFrame);
+  static mozilla::ComputedStyle* FindBackground(const nsIFrame* aForFrame);
+  static nsIFrame* FindBackgroundFrame(const nsIFrame* aForFrame);
 
   /**
    * As FindBackground, but the passed-in frame is known to be a root frame
@@ -301,54 +297,34 @@ struct nsCSSRendering {
   static mozilla::ComputedStyle* FindRootFrameBackground(nsIFrame* aForFrame);
 
   /**
-   * Returns background style information for the canvas.
+   * Find a non-transparent background color on an ancestor, for various
+   * contrast checks. Note that this only accounts for background-color and
+   * might stop at themed frames (depending on the argument), so it might not be
+   * what you want. Note that if we stop at themed frames we might, in fact, end
+   * up returning a transparent color (but then mIsThemed will be set to true).
    *
-   * @param aForFrame
-   *   the frame used to represent the canvas, in the CSS sense (i.e.
-   *   nsCSSRendering::IsCanvasFrame(aForFrame) must be true)
-   * @param aRootElementFrame
-   *   the frame representing the root element of the document
-   * @param aBackground
-   *   contains background style information for the canvas on return
+   * For semi-transparent colors, right now we blend with the default
+   * background-color rather than with all ancestor backgrounds.
+   *
+   * If aPreferBodyToCanvas is true, we prefer the background color of the
+   * <body> frame, even though we found a canvas background, because the body
+   * background color is most likely what will be visible as the background
+   * color of the page, even if the html element has a different background
+   * color which prevents that of the body frame to propagate to the viewport.
    */
-
-  static nsIFrame* FindCanvasBackgroundFrame(const nsIFrame* aForFrame,
-                                             nsIFrame* aRootElementFrame) {
-    MOZ_ASSERT(IsCanvasFrame(aForFrame), "not a canvas frame");
-    if (aRootElementFrame) {
-      return FindBackgroundStyleFrame(aRootElementFrame);
-    }
-
-    // This should always give transparent, so we'll fill it in with the
-    // default color if needed.  This seems to happen a bit while a page is
-    // being loaded.
-    return const_cast<nsIFrame*>(aForFrame);
-  }
-
-  static mozilla::ComputedStyle* FindCanvasBackground(
-      nsIFrame* aForFrame, nsIFrame* aRootElementFrame) {
-    return FindCanvasBackgroundFrame(aForFrame, aRootElementFrame)->Style();
-  }
-
-  /**
-   * Find a frame which draws a non-transparent background, for various contrast
-   * checks. Note that this only accounts for background-color and might stop at
-   * themed frames (depending on the argument), so it might not be what you
-   * want.
-   */
-  struct NonTransparentBackgroundFrame {
-    nsIFrame* mFrame = nullptr;
+  struct EffectiveBackgroundColor {
+    nscolor mColor = 0;
     bool mIsThemed = false;
-    bool mIsForCanvas = false;
   };
-  static NonTransparentBackgroundFrame FindNonTransparentBackgroundFrame(
-      nsIFrame* aFrame, bool aStopAtThemed = true);
+  static EffectiveBackgroundColor FindEffectiveBackgroundColor(
+      nsIFrame* aFrame, bool aStopAtThemed = true,
+      bool aPreferBodyToCanvas = false);
 
   /**
    * Determine the background color to draw taking into account print settings.
    */
   static nscolor DetermineBackgroundColor(nsPresContext* aPresContext,
-                                          mozilla::ComputedStyle* aStyle,
+                                          const mozilla::ComputedStyle* aStyle,
                                           nsIFrame* aFrame,
                                           bool& aDrawBackgroundImage,
                                           bool& aDrawBackgroundColor);
@@ -386,7 +362,7 @@ struct nsCSSRendering {
     nsRect mDirtyRectInAppUnits;
     gfxRect mDirtyRectInDevPx;
 
-    nscoord mRadii[8];
+    nsRectCornerRadii mRadii;
     RectCornerRadii mClippedRadii;
     bool mHasRoundedCorners;
     bool mHasAdditionalBGClipArea;
@@ -398,9 +374,7 @@ struct nsCSSRendering {
     ImageLayerClipState()
         : mHasRoundedCorners(false),
           mHasAdditionalBGClipArea(false),
-          mCustomClip(false) {
-      memset(mRadii, 0, sizeof(nscoord) * 8);
-    }
+          mCustomClip(false) {}
 
     bool IsValid() const;
   };
@@ -442,7 +416,7 @@ struct nsCSSRendering {
      * When this flag is passed, images are downscaled during decode. This
      * is also implied by PAINTBG_TO_WINDOW.
      */
-    PAINTBG_HIGH_QUALITY_SCALING = 0x16,
+    PAINTBG_HIGH_QUALITY_SCALING = 0x10,
   };
 
   struct PaintBGParams {
@@ -502,7 +476,8 @@ struct nsCSSRendering {
    */
   static ImgDrawResult PaintStyleImageLayerWithSC(
       const PaintBGParams& aParams, gfxContext& aRenderingCtx,
-      mozilla::ComputedStyle* mBackgroundSC, const nsStyleBorder& aBorder);
+      const mozilla::ComputedStyle* aBackgroundSC,
+      const nsStyleBorder& aBorder);
 
   static bool CanBuildWebRenderDisplayItemsForStyleImageLayer(
       WebRenderLayerManager* aManager, nsPresContext& aPresCtx,
@@ -579,9 +554,16 @@ struct nsCSSRendering {
       mozilla::Side aStartBevelSide, nscoord aStartBevelOffset,
       mozilla::Side aEndBevelSide, nscoord aEndBevelOffset);
 
-  // NOTE: pt, dirtyRect, lineSize, ascent, offset in the following
-  //       structs are non-rounded device pixels, not app units.
+  // NOTE: pt, dirtyRect, lineSize, ascent, offset, insetLeft, and insetRight in
+  //       the following structs are non-rounded device pixels, not app units.
   struct DecorationRectParams {
+    // Checks if either start or end inset value is negative.
+    // In that situation, text decoration will extend past the edges of the
+    // text.
+    bool HasNegativeInset() const {
+      return insetLeft < 0.0 || insetRight < 0.0;
+    }
+
     // The width [length] and the height [thickness] of the decoration
     // line. This is a "logical" size in textRun orientation, so that
     // for a vertical textrun, width will actually be a physical height;
@@ -611,13 +593,18 @@ struct nsCSSRendering {
     // UNDERLINE or OVERLINE or LINE_THROUGH.
     mozilla::StyleTextDecorationLine decoration =
         mozilla::StyleTextDecorationLine::UNDERLINE;
-    // The style of the decoration line such as
-    // NS_STYLE_TEXT_DECORATION_STYLE_*.
-    uint8_t style = NS_STYLE_TEXT_DECORATION_STYLE_NONE;
+    // The style of the decoration line
+    mozilla::StyleTextDecorationStyle style =
+        mozilla::StyleTextDecorationStyle::None;
+    // Decoration-line start/end inset values for this decoration.
+    // The caller must ensure that the start/end values are translated to
+    // left/right in the line-relative direction.
+    Float insetLeft = 0.0f;
+    Float insetRight = 0.0f;
     bool vertical = false;
     bool sidewaysLeft = false;
     gfxTextRun::Range glyphRange;
-    gfxTextRun::PropertyProvider* provider;
+    gfxTextRun::PropertyProvider* provider = nullptr;
   };
 
   struct PaintDecorationLineParams : DecorationRectParams {
@@ -633,6 +620,11 @@ struct nsCSSRendering {
     // Baseline offset being applied to this text (block-direction adjustment
     // applied to glyph positions when computing skip-ink intercepts).
     Float baselineOffset = 0.0f;
+    // Whether text-decoration-skip-ink behavior is to be supported.
+    bool allowInkSkipping = true;
+
+    mozilla::StyleTextDecorationSkipInk skipInk =
+        mozilla::StyleTextDecorationSkipInk::None;
   };
 
   /**
@@ -757,9 +749,6 @@ struct nsCSSRendering {
    * input:
    *     @param aFrame            the frame which needs the decoration line.
    *     @param aStyle            the style of the complex decoration line
-   *                              NS_STYLE_TEXT_DECORATION_STYLE_DOTTED or
-   *                              NS_STYLE_TEXT_DECORATION_STYLE_DASHED or
-   *                              NS_STYLE_TEXT_DECORATION_STYLE_WAVY.
    *     @param aClippedRect      the clipped rect for the decoration line.
    *                              in other words, visible area of the line.
    *     @param aICoordInFrame  the distance between inline-start edge of aFrame
@@ -767,15 +756,16 @@ struct nsCSSRendering {
    *     @param aCycleLength      the width of one cycle of the line style.
    */
   static Rect ExpandPaintingRectForDecorationLine(
-      nsIFrame* aFrame, const uint8_t aStyle, const Rect& aClippedRect,
-      const Float aICoordInFrame, const Float aCycleLength, bool aVertical);
+      nsIFrame* aFrame, const mozilla::StyleTextDecorationStyle aStyle,
+      const Rect& aClippedRect, const Float aICoordInFrame,
+      const Float aCycleLength, bool aVertical);
 };
 
 /*
  * nsContextBoxBlur
  * Creates an 8-bit alpha channel context for callers to draw in, blurs the
  * contents of that context and applies it as a 1-color mask on a
- * different existing context. Uses gfxAlphaBoxBlur as its back end.
+ * different existing context. Uses gfxGaussianBlur as its back end.
  *
  * You must call Init() first to create a suitable temporary surface to draw
  * on.  You must then draw any desired content onto the given context, then
@@ -790,7 +780,7 @@ class nsContextBoxBlur {
   typedef mozilla::gfx::RectCornerRadii RectCornerRadii;
 
  public:
-  enum { FORCE_MASK = 0x01, DISABLE_HARDWARE_ACCELERATION_BLUR = 0x02 };
+  enum { FORCE_MASK = 0x01 };
   /**
    * Prepares a gfxContext to draw on. Do not call this twice; if you want
    * to get the gfxContext again use GetContext().
@@ -936,8 +926,9 @@ class nsContextBoxBlur {
                                      mozilla::gfx::IntSize& aOutSpreadRadius,
                                      bool aConstrainSpreadRadius = true);
 
-  gfxAlphaBoxBlur mAlphaBoxBlur;
-  RefPtr<gfxContext> mContext;
+  gfxGaussianBlur mGaussianBlur;
+  mozilla::UniquePtr<gfxContext> mOwnedContext;
+  gfxContext* mContext;  // may be either mOwnedContext or mDestinationContext
   gfxContext* mDestinationCtx;
 
   /* This is true if the blur already has it's content transformed
