@@ -5,6 +5,8 @@
 
 #include "nsTableCellFrame.h"
 
+#include <algorithm>
+
 #include "celldata.h"
 #include "gfxContext.h"
 #include "gfxUtils.h"
@@ -14,28 +16,27 @@
 #include "mozilla/StaticPrefs_layout.h"
 #include "mozilla/gfx/2D.h"
 #include "mozilla/gfx/Helpers.h"
-#include "nsTableFrame.h"
-#include "nsTableColFrame.h"
-#include "nsTableRowFrame.h"
-#include "nsTableRowGroupFrame.h"
-#include "nsStyleConsts.h"
-#include "nsPresContext.h"
+#include "nsAttrValueInlines.h"
 #include "nsCSSRendering.h"
+#include "nsDisplayList.h"
+#include "nsGenericHTMLElement.h"
+#include "nsGkAtoms.h"
+#include "nsHTMLParts.h"
 #include "nsIContent.h"
 #include "nsIFrame.h"
 #include "nsIFrameInlines.h"
-#include "nsGenericHTMLElement.h"
-#include "nsAttrValueInlines.h"
-#include "nsHTMLParts.h"
-#include "nsGkAtoms.h"
-#include "nsDisplayList.h"
 #include "nsLayoutUtils.h"
+#include "nsPresContext.h"
+#include "nsStyleConsts.h"
+#include "nsTableColFrame.h"
+#include "nsTableFrame.h"
+#include "nsTableRowFrame.h"
+#include "nsTableRowGroupFrame.h"
 #include "nsTextFrame.h"
-#include <algorithm>
 
 // TABLECELL SELECTION
-#include "nsFrameSelection.h"
 #include "mozilla/LookAndFeel.h"
+#include "nsFrameSelection.h"
 
 #ifdef ACCESSIBILITY
 #  include "nsAccessibilityService.h"
@@ -160,8 +161,7 @@ bool nsTableCellFrame::NeedsToObserve(const ReflowInput& aReflowInput) {
 }
 
 nsresult nsTableCellFrame::AttributeChanged(int32_t aNameSpaceID,
-                                            nsAtom* aAttribute,
-                                            int32_t aModType) {
+                                            nsAtom* aAttribute, AttrModType) {
   // We need to recalculate in this case because of the nowrap quirk in
   // BasicTableLayoutStrategy
   if (aNameSpaceID == kNameSpaceID_None && aAttribute == nsGkAtoms::nowrap &&
@@ -255,63 +255,62 @@ inline nscolor EnsureDifferentColors(nscolor colorA, nscolor colorB) {
 void nsTableCellFrame::DecorateForSelection(DrawTarget* aDrawTarget,
                                             nsPoint aPt) {
   NS_ASSERTION(IsSelected(), "Should only be called for selected cells");
-  int16_t displaySelection;
-  displaySelection = DetermineDisplaySelection();
-  if (displaySelection) {
-    RefPtr<nsFrameSelection> frameSelection = PresShell()->FrameSelection();
-
-    if (frameSelection->IsInTableSelectionMode()) {
-      nscolor bordercolor;
-      if (displaySelection == nsISelectionController::SELECTION_DISABLED) {
-        bordercolor = NS_RGB(176, 176, 176);  // disabled color
-      } else {
-        bordercolor = LookAndFeel::Color(LookAndFeel::ColorID::Highlight, this);
-      }
-      nscoord threePx = nsPresContext::CSSPixelsToAppUnits(3);
-      if ((mRect.width > threePx) && (mRect.height > threePx)) {
-        // compare bordercolor to background-color
-        bordercolor = EnsureDifferentColors(
-            bordercolor, StyleBackground()->BackgroundColor(this));
-
-        int32_t appUnitsPerDevPixel = PresContext()->AppUnitsPerDevPixel();
-        Point devPixelOffset = NSPointToPoint(aPt, appUnitsPerDevPixel);
-
-        AutoRestoreTransform autoRestoreTransform(aDrawTarget);
-        aDrawTarget->SetTransform(
-            aDrawTarget->GetTransform().PreTranslate(devPixelOffset));
-
-        ColorPattern color(ToDeviceColor(bordercolor));
-
-        nscoord onePixel = nsPresContext::CSSPixelsToAppUnits(1);
-
-        StrokeLineWithSnapping(nsPoint(onePixel, 0), nsPoint(mRect.width, 0),
-                               appUnitsPerDevPixel, *aDrawTarget, color);
-        StrokeLineWithSnapping(nsPoint(0, onePixel), nsPoint(0, mRect.height),
-                               appUnitsPerDevPixel, *aDrawTarget, color);
-        StrokeLineWithSnapping(nsPoint(onePixel, mRect.height),
-                               nsPoint(mRect.width, mRect.height),
-                               appUnitsPerDevPixel, *aDrawTarget, color);
-        StrokeLineWithSnapping(nsPoint(mRect.width, onePixel),
-                               nsPoint(mRect.width, mRect.height),
-                               appUnitsPerDevPixel, *aDrawTarget, color);
-        // middle
-        nsRect r(onePixel, onePixel, mRect.width - onePixel,
-                 mRect.height - onePixel);
-        Rect devPixelRect =
-            NSRectToSnappedRect(r, appUnitsPerDevPixel, *aDrawTarget);
-        aDrawTarget->StrokeRect(devPixelRect, color);
-        // shading
-        StrokeLineWithSnapping(
-            nsPoint(2 * onePixel, mRect.height - 2 * onePixel),
-            nsPoint(mRect.width - onePixel, mRect.height - (2 * onePixel)),
-            appUnitsPerDevPixel, *aDrawTarget, color);
-        StrokeLineWithSnapping(
-            nsPoint(mRect.width - (2 * onePixel), 2 * onePixel),
-            nsPoint(mRect.width - (2 * onePixel), mRect.height - onePixel),
-            appUnitsPerDevPixel, *aDrawTarget, color);
-      }
-    }
+  if (!IsSelectable()) {
+    return;
   }
+  RefPtr<nsFrameSelection> frameSelection = PresShell()->FrameSelection();
+  if (!frameSelection->IsInTableSelectionMode()) {
+    return;
+  }
+  nscoord threePx = nsPresContext::CSSPixelsToAppUnits(3);
+  if (mRect.width <= threePx || mRect.height <= threePx) {
+    return;
+  }
+  nscolor bordercolor;
+  if (frameSelection->GetDisplaySelection() ==
+      nsISelectionController::SELECTION_DISABLED) {
+    bordercolor = NS_RGB(176, 176, 176);  // disabled color
+  } else {
+    bordercolor = LookAndFeel::Color(LookAndFeel::ColorID::Highlight, this);
+  }
+  // compare bordercolor to background-color
+  bordercolor = EnsureDifferentColors(bordercolor,
+                                      StyleBackground()->BackgroundColor(this));
+
+  int32_t appUnitsPerDevPixel = PresContext()->AppUnitsPerDevPixel();
+  Point devPixelOffset = NSPointToPoint(aPt, appUnitsPerDevPixel);
+
+  AutoRestoreTransform autoRestoreTransform(aDrawTarget);
+  aDrawTarget->SetTransform(
+      aDrawTarget->GetTransform().PreTranslate(devPixelOffset));
+
+  ColorPattern color(ToDeviceColor(bordercolor));
+
+  nscoord onePixel = nsPresContext::CSSPixelsToAppUnits(1);
+
+  StrokeLineWithSnapping(nsPoint(onePixel, 0), nsPoint(mRect.width, 0),
+                         appUnitsPerDevPixel, *aDrawTarget, color);
+  StrokeLineWithSnapping(nsPoint(0, onePixel), nsPoint(0, mRect.height),
+                         appUnitsPerDevPixel, *aDrawTarget, color);
+  StrokeLineWithSnapping(nsPoint(onePixel, mRect.height),
+                         nsPoint(mRect.width, mRect.height),
+                         appUnitsPerDevPixel, *aDrawTarget, color);
+  StrokeLineWithSnapping(nsPoint(mRect.width, onePixel),
+                         nsPoint(mRect.width, mRect.height),
+                         appUnitsPerDevPixel, *aDrawTarget, color);
+  // middle
+  nsRect r(onePixel, onePixel, mRect.width - onePixel, mRect.height - onePixel);
+  Rect devPixelRect = NSRectToSnappedRect(r, appUnitsPerDevPixel, *aDrawTarget);
+  aDrawTarget->StrokeRect(devPixelRect, color);
+  // shading
+  StrokeLineWithSnapping(
+      nsPoint(2 * onePixel, mRect.height - 2 * onePixel),
+      nsPoint(mRect.width - onePixel, mRect.height - (2 * onePixel)),
+      appUnitsPerDevPixel, *aDrawTarget, color);
+  StrokeLineWithSnapping(
+      nsPoint(mRect.width - (2 * onePixel), 2 * onePixel),
+      nsPoint(mRect.width - (2 * onePixel), mRect.height - onePixel),
+      appUnitsPerDevPixel, *aDrawTarget, color);
 }
 
 void nsTableCellFrame::ProcessBorders(nsTableFrame* aFrame,
@@ -472,18 +471,9 @@ void nsTableCellFrame::AlignChildWithinCell(
   FinishAndStoreOverflow(&reflowOutput);
 
   if (kidPosition != kidRect.Origin(innerWM)) {
-    // Make sure any child views are correctly positioned. We know the inner
-    // table cell won't have a view.
-    nsContainerFrame::PositionChildViews(inner);
-
     // Invalidate new overflow rect.
     inner->InvalidateFrameSubtree();
   }
-  if (HasView()) {
-    nsContainerFrame::SyncFrameViewAfterReflow(PresContext(), this, GetView(),
-                                               reflowOutput.InkOverflow(),
-                                               ReflowChildFlags::Default);
-  }
 }
 
 bool nsTableCellFrame::ComputeCustomOverflow(OverflowAreas& aOverflowAreas) {
@@ -1161,6 +1151,9 @@ void nsTableCellFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
 
   // the 'empty-cells' property has no effect on 'outline'
   DisplayOutline(aBuilder, aLists);
+  if (HidesContent()) {
+    return;
+  }
 
   // The child's background will go in our BorderBackground() list.
   // This isn't a problem since it won't have a real background except for
